==> mule-module-extensions-support-4.11.0-20251111.005452-155-test-sources.jar: org/mule/runtime/module/extension/internal/loader/parser/java/test/MinMuleVersionUtilsTestCase.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.runtime.module.extension.internal.loader.parser.java.test;

import static org.mule.runtime.module.extension.internal.loader.parser.java.utils.MinMuleVersionUtils.declareWithBaselineMmv;
import static org.mule.runtime.module.extension.internal.loader.parser.java.utils.MinMuleVersionUtils.declarationWithMmv;
import static org.mule.runtime.module.extension.internal.loader.parser.java.utils.MinMuleVersionUtils.declarerWithMmv;
import static org.mule.runtime.module.extension.internal.loader.parser.java.utils.ResolvedMinMuleVersion.FIRST_MULE_VERSION;

import static java.util.Optional.empty;
import static java.util.Optional.of;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import org.mule.runtime.api.meta.MuleVersion;
import org.mule.runtime.api.meta.model.declaration.fluent.HasMinMuleVersionDeclarer;
import org.mule.runtime.api.meta.model.declaration.fluent.OperationDeclarer;
import org.mule.runtime.api.meta.model.declaration.fluent.WithMinMuleVersionDeclaration;
import org.mule.runtime.extension.api.loader.ExtensionLoadingContext;
import org.mule.runtime.extension.api.loader.parser.MinMuleVersionParser;
import org.mule.runtime.module.extension.internal.loader.parser.java.utils.ResolvedMinMuleVersion;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class MinMuleVersionUtilsTestCase {

  private static final MuleVersion TEST_VERSION_4_6 = new MuleVersion("4.6");
  private static final MuleVersion TEST_VERSION_4_4 = new MuleVersion("4.4");
  private static final MuleVersion TEST_VERSION_4_8 = new MuleVersion("4.8");

  private ExtensionLoadingContext contextResolvingMinMuleVersion;
  private ExtensionLoadingContext contextNotResolvingMinMuleVersion;
  private HasMinMuleVersionDeclarer<?> mockDeclarer;
  private WithMinMuleVersionDeclaration mockDeclaration;
  private MinMuleVersionParser resolvedMMV;

  @BeforeEach
  void setUp() {
    contextResolvingMinMuleVersion = MinMuleVersionTestUtils.ctxResolvingMinMuleVersion();
    contextNotResolvingMinMuleVersion = mock(ExtensionLoadingContext.class);
    when(contextNotResolvingMinMuleVersion.isResolveMinMuleVersion()).thenReturn(false);
    mockDeclarer = mock(OperationDeclarer.class);
    mockDeclaration = mock(WithMinMuleVersionDeclaration.class);
    resolvedMMV = new ResolvedMinMuleVersion("testComponent", TEST_VERSION_4_6,
                                             "Test component has min mule version 4.6 for testing purposes.");
  }

  @Test
  void testDeclarerWithMmv() {
    declarerWithMmv(mockDeclarer, resolvedMMV);
    verify(mockDeclarer).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclarationWithMmv() {
    declarationWithMmv(mockDeclaration, resolvedMMV);
    verify(mockDeclaration).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenContextNotResolving() {
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclarer, empty(), contextNotResolvingMinMuleVersion);
    verify(mockDeclarer, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenContextResolving() {
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclarer, empty(), contextResolvingMinMuleVersion);
    verify(mockDeclarer).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenNoResolvedMMV() {
    declareWithBaselineMmv(() -> empty(), mockDeclarer, empty(), contextResolvingMinMuleVersion);
    verify(mockDeclarer, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenBaselineLower() {
    // Baseline (4.4) is lower than element MMV (4.6), so element MMV should be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclarer, of(TEST_VERSION_4_4),
                           contextResolvingMinMuleVersion);
    verify(mockDeclarer).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenBaselineHigher() {
    // Baseline (4.8) is higher than element MMV (4.6), so element MMV should NOT be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclarer, of(TEST_VERSION_4_8),
                           contextResolvingMinMuleVersion);
    verify(mockDeclarer, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenBaselineEqual() {
    // Baseline (4.6) is equal to element MMV (4.6), baseline is not "newer than", so element MMV should be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclarer, of(TEST_VERSION_4_6),
                           contextResolvingMinMuleVersion);
    verify(mockDeclarer).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenContextNotResolving() {
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclaration, empty(), contextNotResolvingMinMuleVersion);
    verify(mockDeclaration, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenContextResolving() {
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclaration, empty(), contextResolvingMinMuleVersion);
    verify(mockDeclaration).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenNoResolvedMMV() {
    declareWithBaselineMmv(() -> empty(), mockDeclaration, empty(), contextResolvingMinMuleVersion);
    verify(mockDeclaration, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenBaselineLower() {
    // Baseline (4.4) is lower than element MMV (4.6), so element MMV should be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclaration, of(TEST_VERSION_4_4),
                           contextResolvingMinMuleVersion);
    verify(mockDeclaration).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenBaselineHigher() {
    // Baseline (4.8) is higher than element MMV (4.6), so element MMV should NOT be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclaration, of(TEST_VERSION_4_8),
                           contextResolvingMinMuleVersion);
    verify(mockDeclaration, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenBaselineEqual() {
    // Baseline (4.6) is equal to element MMV (4.6), baseline is not "newer than", so element MMV should be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclaration, of(TEST_VERSION_4_6),
                           contextResolvingMinMuleVersion);
    verify(mockDeclaration).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWithDefaultVersion() {
    // If resolved MMV is FIRST_MULE_VERSION (4.1.1) and there's no baseline, it should be applied
    ResolvedMinMuleVersion defaultMMV = new ResolvedMinMuleVersion("testComponent", FIRST_MULE_VERSION,
                                                                   "Test component has default min mule version.");
    declareWithBaselineMmv(() -> of(defaultMMV), mockDeclaration, empty(), contextResolvingMinMuleVersion);
    verify(mockDeclaration).withMinMuleVersion(FIRST_MULE_VERSION);
  }
}

==> mule-module-extensions-support-4.11.0-20251203.140611-181-test-sources.jar: org/mule/runtime/module/extension/internal/loader/parser/java/test/MinMuleVersionUtilsTestCase.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.runtime.module.extension.internal.loader.parser.java.test;

import static org.mule.runtime.module.extension.internal.loader.parser.java.utils.MinMuleVersionUtils.declarationWithMmv;
import static org.mule.runtime.module.extension.internal.loader.parser.java.utils.MinMuleVersionUtils.declareWithBaselineMmv;
import static org.mule.runtime.module.extension.internal.loader.parser.java.utils.MinMuleVersionUtils.declarerWithMmv;
import static org.mule.runtime.module.extension.internal.loader.parser.java.utils.MinMuleVersionUtils.resolveRouterMinMuleVersion;
import static org.mule.runtime.module.extension.internal.loader.parser.java.utils.ResolvedMinMuleVersion.FIRST_MULE_VERSION;

import static java.util.Arrays.asList;
import static java.util.Collections.emptyList;
import static java.util.Optional.empty;
import static java.util.Optional.of;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import org.mule.runtime.api.meta.MuleVersion;
import org.mule.runtime.api.meta.model.declaration.fluent.HasMinMuleVersionDeclarer;
import org.mule.runtime.api.meta.model.declaration.fluent.OperationDeclarer;
import org.mule.runtime.api.meta.model.declaration.fluent.WithMinMuleVersionDeclaration;
import org.mule.runtime.extension.api.loader.ExtensionLoadingContext;
import org.mule.runtime.extension.api.loader.parser.MinMuleVersionParser;
import org.mule.runtime.module.extension.api.loader.java.type.ExtensionParameter;
import org.mule.runtime.module.extension.api.loader.java.type.MethodElement;
import org.mule.runtime.module.extension.api.loader.java.type.Type;
import org.mule.runtime.module.extension.internal.loader.parser.java.utils.ResolvedMinMuleVersion;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class MinMuleVersionUtilsTestCase {

  private static final MuleVersion TEST_VERSION_4_6 = new MuleVersion("4.6");
  private static final MuleVersion TEST_VERSION_4_4 = new MuleVersion("4.4");
  private static final MuleVersion TEST_VERSION_4_8 = new MuleVersion("4.8");

  private ExtensionLoadingContext contextResolvingMinMuleVersion;
  private ExtensionLoadingContext contextNotResolvingMinMuleVersion;
  private HasMinMuleVersionDeclarer<?> mockDeclarer;
  private WithMinMuleVersionDeclaration mockDeclaration;
  private MinMuleVersionParser resolvedMMV;

  @BeforeEach
  void setUp() {
    contextResolvingMinMuleVersion = MinMuleVersionTestUtils.ctxResolvingMinMuleVersion();
    contextNotResolvingMinMuleVersion = mock(ExtensionLoadingContext.class);
    when(contextNotResolvingMinMuleVersion.isResolveMinMuleVersion()).thenReturn(false);
    mockDeclarer = mock(OperationDeclarer.class);
    mockDeclaration = mock(WithMinMuleVersionDeclaration.class);
    resolvedMMV = new ResolvedMinMuleVersion("testComponent", TEST_VERSION_4_6,
                                             "Test component has min mule version 4.6 for testing purposes.");
  }

  @Test
  void testDeclarerWithMmv() {
    declarerWithMmv(mockDeclarer, resolvedMMV);
    verify(mockDeclarer).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclarationWithMmv() {
    declarationWithMmv(mockDeclaration, resolvedMMV);
    verify(mockDeclaration).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenContextNotResolving() {
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclarer, empty(), contextNotResolvingMinMuleVersion);
    verify(mockDeclarer, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenContextResolving() {
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclarer, empty(), contextResolvingMinMuleVersion);
    verify(mockDeclarer).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenNoResolvedMMV() {
    declareWithBaselineMmv(() -> empty(), mockDeclarer, empty(), contextResolvingMinMuleVersion);
    verify(mockDeclarer, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenBaselineLower() {
    // Baseline (4.4) is lower than element MMV (4.6), so element MMV should be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclarer, of(TEST_VERSION_4_4),
                           contextResolvingMinMuleVersion);
    verify(mockDeclarer).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenBaselineHigher() {
    // Baseline (4.8) is higher than element MMV (4.6), so element MMV should NOT be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclarer, of(TEST_VERSION_4_8),
                           contextResolvingMinMuleVersion);
    verify(mockDeclarer, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarerWhenBaselineEqual() {
    // Baseline (4.6) is equal to element MMV (4.6), baseline is not "newer than", so element MMV should be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclarer, of(TEST_VERSION_4_6),
                           contextResolvingMinMuleVersion);
    verify(mockDeclarer).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenContextNotResolving() {
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclaration, empty(), contextNotResolvingMinMuleVersion);
    verify(mockDeclaration, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenContextResolving() {
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclaration, empty(), contextResolvingMinMuleVersion);
    verify(mockDeclaration).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenNoResolvedMMV() {
    declareWithBaselineMmv(() -> empty(), mockDeclaration, empty(), contextResolvingMinMuleVersion);
    verify(mockDeclaration, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenBaselineLower() {
    // Baseline (4.4) is lower than element MMV (4.6), so element MMV should be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclaration, of(TEST_VERSION_4_4),
                           contextResolvingMinMuleVersion);
    verify(mockDeclaration).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenBaselineHigher() {
    // Baseline (4.8) is higher than element MMV (4.6), so element MMV should NOT be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclaration, of(TEST_VERSION_4_8),
                           contextResolvingMinMuleVersion);
    verify(mockDeclaration, never()).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWhenBaselineEqual() {
    // Baseline (4.6) is equal to element MMV (4.6), baseline is not "newer than", so element MMV should be applied
    declareWithBaselineMmv(() -> of(resolvedMMV), mockDeclaration, of(TEST_VERSION_4_6),
                           contextResolvingMinMuleVersion);
    verify(mockDeclaration).withMinMuleVersion(TEST_VERSION_4_6);
  }

  @Test
  void testDeclareWithBaselineMmvDeclarationWithDefaultVersion() {
    // If resolved MMV is FIRST_MULE_VERSION (4.1.1) and there's no baseline, it should be applied
    ResolvedMinMuleVersion defaultMMV = new ResolvedMinMuleVersion("testComponent", FIRST_MULE_VERSION,
                                                                   "Test component has default min mule version.");
    declareWithBaselineMmv(() -> of(defaultMMV), mockDeclaration, empty(), contextResolvingMinMuleVersion);
    verify(mockDeclaration).withMinMuleVersion(FIRST_MULE_VERSION);
  }

  @Test
  void resolveRouterMinMuleVersionWithStaticRoutes() {
    // Router with static routes (non-List type) should return the original MMV
    MethodElement<?> router = createMockRouter("testRouter");
    ExtensionParameter route1 = createMockRoute(String.class);
    ExtensionParameter route2 = createMockRoute(Integer.class);
    ResolvedMinMuleVersion inputMMV = createRouterMMV("testRouter", TEST_VERSION_4_4);

    ResolvedMinMuleVersion result = resolveRouterMinMuleVersion(router, inputMMV, asList(route1, route2));

    assertThat(result.getMinMuleVersion(), is(TEST_VERSION_4_4));
  }

  @Test
  void resolveRouterMinMuleVersionWithDynamicRoutes() {
    // Router with dynamic routes (List type) should return v4_10_0
    MethodElement<?> router = createMockRouter("testRouter");
    ExtensionParameter route1 = createMockRoute(List.class);
    ResolvedMinMuleVersion inputMMV = createRouterMMV("testRouter", TEST_VERSION_4_4);

    ResolvedMinMuleVersion result = resolveRouterMinMuleVersion(router, inputMMV, asList(route1));

    assertThat(result.getMinMuleVersion(), is(new MuleVersion("4.10.0")));
  }

  @Test
  void resolveRouterMinMuleVersionWithEmptyRoutes() {
    // Router with no routes should return the original MMV
    MethodElement<?> router = createMockRouter("testRouter");
    ResolvedMinMuleVersion inputMMV = createRouterMMV("testRouter", TEST_VERSION_4_4);

    ResolvedMinMuleVersion result = resolveRouterMinMuleVersion(router, inputMMV, emptyList());

    assertThat(result.getMinMuleVersion(), is(TEST_VERSION_4_4));
  }

  @Test
  void resolveRouterMinMuleVersionWithMixedRoutes() {
    // Router with mixed routes (some List, some not) should return v4_10_0 if any route is a List
    MethodElement<?> router = createMockRouter("testRouter");
    ExtensionParameter route1 = createMockRoute(String.class);
    ExtensionParameter route2 = createMockRoute(List.class);
    ResolvedMinMuleVersion inputMMV = createRouterMMV("testRouter", TEST_VERSION_4_4);

    ResolvedMinMuleVersion result = resolveRouterMinMuleVersion(router, inputMMV, asList(route1, route2));

    assertThat(result.getMinMuleVersion(), is(new MuleVersion("4.10.0")));
  }

  @Test
  void resolveRouterMinMuleVersionWithHigherInputMMV() {
    // If input MMV is already higher than 4.10.0, it should be returned
    MethodElement<?> router = createMockRouter("testRouter");
    ExtensionParameter route1 = createMockRoute(List.class);
    ResolvedMinMuleVersion inputMMV = createRouterMMV("testRouter", new MuleVersion("4.12.0"));

    ResolvedMinMuleVersion result = resolveRouterMinMuleVersion(router, inputMMV, asList(route1));

    assertThat(result.getMinMuleVersion(), is(new MuleVersion("4.12.0")));
  }

  // Helper methods for router tests

  private MethodElement<?> createMockRouter(String name) {
    MethodElement<?> router = mock(MethodElement.class);
    when(router.getName()).thenReturn(name);
    return router;
  }

  private ExtensionParameter createMockRoute(Class<?> declaringClass) {
    ExtensionParameter route = mock(ExtensionParameter.class);
    Type routeType = mock(Type.class);
    when(route.getType()).thenReturn(routeType);
    when(routeType.getDeclaringClass()).thenReturn(of(declaringClass));
    return route;
  }

  private ResolvedMinMuleVersion createRouterMMV(String name, MuleVersion version) {
    return new ResolvedMinMuleVersion(name, version,
                                      "Test router has min mule version " + version + ".");
  }
}
